#pragma once

#include "Utility.hpp"
#include "ctll/fixed_string.hpp"
#include <cpr/cpr.h>
#include <ctll.hpp>
#include <ctre.hpp>
#include <glaze/glaze.hpp>
#include <stdexcept>
#include <string>

// Example regex patern: (?<=protein\s)(\d+)\s(\w)

struct Data {
  double cals{};
  double proteins{};
  double carbs{};
  double fats{};
};

class WolframAlpha {
public:
  WolframAlpha(const std::string& query, double amount) : m_plaintext{getData(query)}, m_amount{amount / 100} {}

  //debug
  std::string getPlain() {
    return m_plaintext;
  }

  Data getNutrition() {
    return m_data;
  }

private:
  void getMacro() {
    static constexpr ctll::fixed_string calories_regex{R"((?<=total calories\s)\d+)"};
    static constexpr ctll::fixed_string proteins_regex{R"((?<=protein\s)(\d+)\s(\w))"};
    static constexpr ctll::fixed_string carbs_regex{R"((?<=total carbohydrates\s)(\d+)\s(\w))"};
    static constexpr ctll::fixed_string fats_regex{R"((?<=total fat\s)(\d+)\s(\w))"};

    const auto                          convertToGrams = [this](const auto& match, const std::string& unit) -> double {
      const double value = (unit == "g") ? match.to_number() : match.to_number() / 1000;
      return value * m_amount;
    };

    if (auto match = ctre::search<proteins_regex>(m_plaintext); match) {
      m_data.proteins = convertToGrams(match.get<1>(), match.get<2>().to_string());
    }

    if (auto match = ctre::search<carbs_regex>(m_plaintext); match) {
      m_data.carbs = convertToGrams(match.get<1>(), match.get<2>().to_string());
    }

    if (auto match = ctre::search<fats_regex>(m_plaintext); match) {
      m_data.fats = convertToGrams(match.get<1>(), match.get<2>().to_string());
    }

    if (auto match = ctre::search<calories_regex>(m_plaintext); match) {
      m_data.cals = match.get<0>().to_number();
    }
  }
  std::string getData(const std::string& query) {
    const auto    WOLFRAM_KEY = utl::getEnv("NUTRI_WA_KEY");
    cpr::Response response    = Get(cpr::Url{"https://api.wolframalpha.com/v2/query"},
                                    cpr::Parameters{{"input", query + "100g nutrition"}, {"appid", WOLFRAM_KEY}, {"output", "JSON"}, {"format", "plaintext"}});

    glz::json_t   json{};
    // Pretty sure this is wrong and i always is false
    if (glz::read_json(json, response.text)) {
      throw std::runtime_error("Failed to parse the JSON");
    } else {
      return json["queryresult"]["pods"][1]["subpods"][0]["plaintext"].get<std::string>();
    }
  }

  Data        m_data{};
  std::string m_pod{};
  std::string m_plaintext{};
  double      m_amount{};
};
